/*
* Copyright (c) 2015 Razeware LLC
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*/

import MediaPlayer
import UIKit

class ViewController: UIViewController {
  // MARK: - Properties
  @IBOutlet var albumLabel: UILabel!
  @IBOutlet var artistLabel: UILabel!
  @IBOutlet var nextButton: UIButton!
  @IBOutlet var playPauseButton: UIButton!
  @IBOutlet var previousButton: UIButton!
  @IBOutlet var progressView: ProgressBar!
  @IBOutlet var songLabel: UILabel!
  @IBOutlet var stopButton: UIButton!
  @IBOutlet var tableView: UITableView!
  
  var mediaPlayer:MPMusicPlayerController = MPMusicPlayerController.applicationMusicPlayer()
  
  var mediaItemsArray:[MPMediaItem] {
    get {
      let mediaQuery: MPMediaQuery = MPMediaQuery()
      mediaQuery.addFilterPredicate(MPMediaPropertyPredicate(value: MPMediaType.Music.rawValue , forProperty: MPMediaItemPropertyMediaType))
      
      return mediaQuery.items!
    }
  }
  
  // MARK: - View Controller
  override func viewDidLoad() {
    super.viewDidLoad()
    
    updatePlayerLabels("", artist: "", album: "")
    NSTimer.scheduledTimerWithTimeInterval(0.01, target: self, selector: #selector(ViewController.refresh), userInfo: nil, repeats: true)
  }
  
  private func updatePlayerLabels(song: String, artist: String, album: String) {
    songLabel.text = song
    artistLabel.text = artist
    albumLabel.text = album
  }
  
  func refresh() {
    if let currentSong = mediaPlayer.nowPlayingItem {
      let title = currentSong.title ?? ""
      let artist = currentSong.artist ?? ""
      let album = currentSong.albumTitle ?? ""
      
      if (mediaPlayer.currentPlaybackTime /
                currentSong.playbackDuration) .isNormal {
        progressView.progress = CGFloat(mediaPlayer.currentPlaybackTime /
                      currentSong.playbackDuration)
      } else {
        progressView.progress = 0.0
      }
      
      updatePlayerLabels(title, artist: artist, album: album)
    } else {
      updatePlayerLabels("", artist: "", album: "")
      progressView.progress = 0.0
    }
  }
  
  // MARK: - IBActions
  @IBAction func nextTapped() {
    if (mediaPlayer.nowPlayingItem != nil) {
      mediaPlayer.skipToNextItem()
    }
  }
  
  @IBAction func playPauseTapped() {
    if mediaPlayer.playbackState == .Paused {
      mediaPlayer.play()
    } else {
      mediaPlayer.pause()
    }
  }
  
  @IBAction func previousTapped() {
    if (mediaPlayer.nowPlayingItem != nil) {
      mediaPlayer.skipToPreviousItem()
    }
  }
  
  @IBAction func stopTapped() {
    mediaPlayer.stop()
  }
  

}

// MARK: - Table View Data Source
extension ViewController: UITableViewDataSource {

  func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
    let currentSong = mediaItemsArray[indexPath.row]
    
    let cell = UITableViewCell(style: .Subtitle, reuseIdentifier: "SongCell")
    
    cell.imageView?.contentMode = .ScaleAspectFit
    cell.imageView?.clipsToBounds = true
    
    cell.textLabel?.text = currentSong.title
    cell.detailTextLabel?.text = currentSong.albumArtist
    if let size = cell.imageView?.bounds.size {
      cell.imageView?.image = currentSong.artwork?.imageWithSize(size)
    }
    return cell
  }
  
  func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
    return mediaItemsArray.count
  }
  
}

// MARK: - Table View Delegate
extension ViewController: UITableViewDelegate {

  func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
    tableView.deselectRowAtIndexPath(indexPath, animated: true)
    
    let collection = MPMediaItemCollection(items: mediaItemsArray)
    let currentSong = mediaItemsArray[indexPath.row]
    
    mediaPlayer.setQueueWithItemCollection(collection)
    mediaPlayer.stop()
    mediaPlayer.nowPlayingItem = currentSong
    mediaPlayer.play()
  }
}

